@@ -27,16 +27,12 @@ mod state;
 use std::any::Any;
 use std::collections::{HashSet, VecDeque};
 use std::convert::TryFrom;
-use std::path::Path;
 use std::sync::{Arc, Mutex, RwLock};
 use std::time::Duration;
 
 use cylinder::Verifier as SignatureVerifier;
 use protobuf::Message;
-#[cfg(feature = "receipt-store")]
 use sawtooth::receipt::store::ReceiptStore;
-#[cfg(not(feature = "receipt-store"))]
-use sawtooth::store::receipt_store::TransactionReceiptStore;
 use splinter::{
     consensus::{Proposal, ProposalUpdate},
     service::{
@@ -49,14 +45,18 @@ use transact::{
     protos::{FromBytes, IntoBytes},
 };
 
+#[cfg(feature = "database-support")]
+use crate::store::CommitHashStore;
+
 use super::protos::scabbard::{ScabbardMessage, ScabbardMessage_Type};
 
 use consensus::ScabbardConsensusManager;
 use error::ScabbardError;
-#[cfg(feature = "service-arg-validation")]
 pub use factory::ScabbardArgValidator;
 pub use factory::{ScabbardFactory, ScabbardFactoryBuilder, ScabbardStorageConfiguration};
 use shared::ScabbardShared;
+#[cfg(feature = "database-support")]
+use state::merkle_state::MerkleState;
 pub use state::{
     BatchInfo, BatchInfoIter, BatchStatus, Events, StateChange, StateChangeEvent, StateIter,
 };
@@ -86,10 +86,6 @@ impl TryFrom<Option<&str>> for ScabbardVersion {
     }
 }
 
-#[cfg(not(feature = "receipt-store"))]
-type ScabbardReceiptStore = Arc<RwLock<TransactionReceiptStore>>;
-
-#[cfg(feature = "receipt-store")]
 type ScabbardReceiptStore = Arc<RwLock<dyn ReceiptStore>>;
 
 /// A service for running Sawtooth Sabre smart contracts with two-phase commit consensus.
@@ -116,10 +112,14 @@ impl Scabbard {
         version: ScabbardVersion,
         // List of other scabbard services on the same circuit that this service shares state with
         peer_services: HashSet<String>,
+        #[cfg(not(feature = "database-support"))]
         // The directory in which to create sabre's LMDB database
-        state_db_path: &Path,
+        state_db_path: &std::path::Path,
+        #[cfg(not(feature = "database-support"))]
         // The size of sabre's LMDB database
         state_db_size: usize,
+        #[cfg(feature = "database-support")] merkle_state: MerkleState,
+        #[cfg(feature = "database-support")] commit_hash_store: Box<dyn CommitHashStore>,
         receipt_store: ScabbardReceiptStore,
         purge_fn: Box<dyn Fn() -> Result<(), splinter::error::InternalError> + Send + Sync>,
         signature_verifier: Box<dyn SignatureVerifier>,
@@ -140,6 +140,7 @@ impl Scabbard {
             version,
         );
 
+        #[cfg(not(feature = "database-support"))]
         let state = ScabbardState::new(
             state_db_path,
             state_db_size,
@@ -152,6 +153,19 @@ impl Scabbard {
         )
         .map_err(|err| ScabbardError::InitializationFailed(Box::new(err)))?;
 
+        #[cfg(feature = "database-support")]
+        let state = ScabbardState::new(
+            merkle_state,
+            commit_hash_store,
+            receipt_store,
+            #[cfg(feature = "metrics")]
+            service_id.clone(),
+            #[cfg(feature = "metrics")]
+            circuit_id.to_string(),
+            admin_keys,
+        )
+        .map_err(|err| ScabbardError::InitializationFailed(Box::new(err)))?;
+
         let coordinator_timeout =
             coordinator_timeout.unwrap_or_else(|| Duration::from_secs(DEFAULT_COORDINATOR_TIMEOUT));
 
@@ -518,45 +532,58 @@ pub mod tests {
     use super::*;
 
     use std::error::Error;
-    use std::path::PathBuf;
 
     use cylinder::{secp256k1::Secp256k1Context, VerifierFactory};
-    #[cfg(feature = "receipt-store")]
     use sawtooth::receipt::store::{ReceiptIter, ReceiptStoreError};
-    #[cfg(not(feature = "receipt-store"))]
-    use sawtooth::store::lmdb::LmdbOrderedStore;
     use splinter::service::{
         ServiceConnectionError, ServiceDisconnectionError, ServiceMessageContext,
         ServiceNetworkSender, ServiceSendError,
     };
+    #[cfg(not(feature = "database-support"))]
     use tempdir::TempDir;
-    #[cfg(feature = "receipt-store")]
     use transact::protocol::receipt::TransactionReceipt;
+    #[cfg(feature = "database-support")]
+    use transact::{
+        database::{btree::BTreeDatabase, Database},
+        state::merkle::INDEXES,
+    };
 
+    #[cfg(not(feature = "database-support"))]
     use crate::service::factory::compute_db_path;
+    #[cfg(feature = "database-support")]
+    use crate::service::state::merkle_state::MerkleStateConfig;
+    #[cfg(feature = "database-support")]
+    use crate::store::transact::{TransactCommitHashStore, CURRENT_STATE_ROOT_INDEX};
 
+    #[cfg(not(feature = "database-support"))]
     const MOCK_CIRCUIT_ID: &str = "abcde-01234";
+    #[cfg(not(feature = "database-support"))]
     const MOCK_SERVICE_ID: &str = "ABCD";
+    #[cfg(not(feature = "database-support"))]
     const TEMP_DB_SIZE: usize = 1 << 30; // 1024 ** 3
 
     /// Tests that a new scabbard service is properly instantiated.
     #[test]
     fn new_scabbard() {
+        #[cfg(not(feature = "database-support"))]
         let paths = StatePaths::new("new_scabbard");
 
+        #[cfg(feature = "database-support")]
+        let (merkle_state, commit_hash_store) = create_merkle_state_and_commit_hash_store();
+
         let service = Scabbard::new(
             "new_scabbard".into(),
             "test_circuit",
             ScabbardVersion::V1,
             HashSet::new(),
+            #[cfg(not(feature = "database-support"))]
             paths.state_db_path.as_path(),
+            #[cfg(not(feature = "database-support"))]
             TEMP_DB_SIZE,
-            #[cfg(not(feature = "receipt-store"))]
-            Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
-                LmdbOrderedStore::new(&paths.receipt_db_path, Some(TEMP_DB_SIZE))
-                    .expect("Unable to make lmdb store"),
-            )))),
-            #[cfg(feature = "receipt-store")]
+            #[cfg(feature = "database-support")]
+            merkle_state,
+            #[cfg(feature = "database-support")]
+            commit_hash_store,
             Arc::new(RwLock::new(MockReceiptStore)),
             Box::new(|| Ok(())),
             Secp256k1Context::new().new_verifier(),
@@ -572,21 +599,24 @@ pub mod tests {
     /// will hang if the thread does not get shutdown correctly.
     #[test]
     fn thread_cleanup() {
+        #[cfg(not(feature = "database-support"))]
         let paths = StatePaths::new("new_scabbard");
+        #[cfg(feature = "database-support")]
+        let (merkle_state, commit_hash_store) = create_merkle_state_and_commit_hash_store();
 
         let mut service = Scabbard::new(
             "thread_cleanup".into(),
             "test_circuit",
             ScabbardVersion::V1,
             HashSet::new(),
+            #[cfg(not(feature = "database-support"))]
             paths.state_db_path.as_path(),
+            #[cfg(not(feature = "database-support"))]
             TEMP_DB_SIZE,
-            #[cfg(not(feature = "receipt-store"))]
-            Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
-                LmdbOrderedStore::new(&paths.receipt_db_path, Some(TEMP_DB_SIZE))
-                    .expect("Unable to make lmdb store"),
-            )))),
-            #[cfg(feature = "receipt-store")]
+            #[cfg(feature = "database-support")]
+            merkle_state,
+            #[cfg(feature = "database-support")]
+            commit_hash_store,
             Arc::new(RwLock::new(MockReceiptStore)),
             Box::new(|| Ok(())),
             Secp256k1Context::new().new_verifier(),
@@ -602,21 +632,24 @@ pub mod tests {
     /// Tests that the service properly connects and disconnects using the network registry.
     #[test]
     fn connect_and_disconnect() {
+        #[cfg(not(feature = "database-support"))]
         let paths = StatePaths::new("new_scabbard");
 
+        #[cfg(feature = "database-support")]
+        let (merkle_state, commit_hash_store) = create_merkle_state_and_commit_hash_store();
         let mut service = Scabbard::new(
             "connect_and_disconnect".into(),
             "test_circuit",
             ScabbardVersion::V1,
             HashSet::new(),
+            #[cfg(not(feature = "database-support"))]
             paths.state_db_path.as_path(),
+            #[cfg(not(feature = "database-support"))]
             TEMP_DB_SIZE,
-            #[cfg(not(feature = "receipt-store"))]
-            Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
-                LmdbOrderedStore::new(&paths.receipt_db_path, Some(TEMP_DB_SIZE))
-                    .expect("Unable to make lmdb store"),
-            )))),
-            #[cfg(feature = "receipt-store")]
+            #[cfg(feature = "database-support")]
+            merkle_state,
+            #[cfg(feature = "database-support")]
+            commit_hash_store,
             Arc::new(RwLock::new(MockReceiptStore)),
             Box::new(|| Ok(())),
             Secp256k1Context::new().new_verifier(),
@@ -627,14 +660,26 @@ pub mod tests {
         test_connect_and_disconnect(&mut service);
     }
 
+    #[cfg(feature = "database-support")]
+    fn create_merkle_state_and_commit_hash_store(
+    ) -> (MerkleState, Box<dyn CommitHashStore + Send + Sync>) {
+        let mut indexes = INDEXES.to_vec();
+        indexes.push(CURRENT_STATE_ROOT_INDEX);
+        let db = BTreeDatabase::new(&indexes);
+        let merkle_state = MerkleState::new(MerkleStateConfig::key_value(db.clone_box()))
+            .expect("Unable to create merkle state");
+        let commit_hash_store = TransactCommitHashStore::new(db);
+        (merkle_state, Box::new(commit_hash_store))
+    }
+
+    #[cfg(not(feature = "database-support"))]
     struct StatePaths {
         // this is deleted when dropped
         _temp_dir: TempDir,
-        pub state_db_path: PathBuf,
-        #[cfg(not(feature = "receipt-store"))]
-        pub receipt_db_path: PathBuf,
+        pub state_db_path: std::path::PathBuf,
     }
 
+    #[cfg(not(feature = "database-support"))]
     impl StatePaths {
         fn new(prefix: &str) -> Self {
             let temp_dir = TempDir::new(prefix).expect("Failed to create temp dir");
@@ -643,19 +688,9 @@ pub mod tests {
             let state_db_path =
                 compute_db_path(MOCK_SERVICE_ID, MOCK_CIRCUIT_ID, temp_dir.path(), "-state")
                     .expect("Failed to compute DB paths");
-            #[cfg(not(feature = "receipt-store"))]
-            let receipt_db_path = compute_db_path(
-                MOCK_SERVICE_ID,
-                MOCK_CIRCUIT_ID,
-                temp_dir.path(),
-                "-receipts",
-            )
-            .expect("Failed to compute DB paths");
             Self {
                 _temp_dir: temp_dir,
                 state_db_path,
-                #[cfg(not(feature = "receipt-store"))]
-                receipt_db_path,
             }
         }
     }
@@ -794,10 +829,8 @@ pub mod tests {
         }
     }
 
-    #[cfg(feature = "receipt-store")]
     struct MockReceiptStore;
 
-    #[cfg(feature = "receipt-store")]
     impl ReceiptStore for MockReceiptStore {
         fn get_txn_receipt_by_id(
             &self,
